import java.util.Random;

public class PriorityQueueTester {
    static Random rand = new Random(10);
    static final int SIZE = 20;
    
    public static void main(String[] args) {
	SimplePriorityQueue<Integer> q = new LinearPriorityQueue<Integer>();

	int[] vals = getVals();
	int[] prior = getPriorities();

	System.out.println("Testing insert method...");
	if (testInsert(q, vals, prior)) {
	    System.out.println("...test passed!");
	} else {
	    System.out.println("...test failed.");
	}

	System.out.println("Contents: " + q);

	System.out.println("Testing remove method...");
	if (testRemove(q, vals, prior)) {
	    System.out.println("...test passed!");
	} else {
	    System.out.println("...test failed.");
	}

	System.out.println("Contents: " + q);	
    }

    // return an array of random integer values sorted in increasing
    // order
    static int[] getVals() {
	int[] vals = new int[SIZE];
	int cur = 0;
	for (int i = 0; i < SIZE; ++i) {
	    cur += 1 + rand.nextInt(10);
	    vals[i] = cur;
	}
	return vals;
    }

    static int[] getPriorities() {
	int[] p = new int[SIZE];

	// use fisher-yates to shuffle priorities
	for (int i = 1; i < SIZE; ++i) {
	    int j = rand.nextInt(i+1);
	    p[i] = p[j];
	    p[j] = i;
	}

	return p;
    }

    static int[] getInversePriorities(int[] p) {
	int[] inv = new int[SIZE];

	for (int i = 0; i < p.length; ++i) {
	    inv[p[i]] = i;
	}

	return inv;
    }
	
    static boolean testInsert(SimplePriorityQueue<Integer> q, int[] vals, int[] p) {
	
	int minP = Integer.MAX_VALUE;
	int minV = 0;
	
	for (int i = vals.length - 1; i >= 0; --i) {
	    System.out.println("    adding " + vals[i] + " with priority " + p[i]);
	    q.insert(p[i], vals[i]);

	    if (p[i] < minP) {
		minP = p[i];
		minV = vals[i];
	    }

	    if (q.min() == null || !q.min().equals(minV)) {
		System.out.println("    ...incorrect min after inserting " + vals[i]);
		return false;
	    }

	    if (q.size() != vals.length - i) {
		System.out.println("    ...incrrect size after inserting " + vals[i]);
		return false;
	    }
	}

	return true;
    }

    static boolean testRemove(SimplePriorityQueue<Integer> q, int[] vals, int[] prior) {
	int[] inv = getInversePriorities(prior);
	for (int i = 0; i < vals.length; ++i) {
	    long val = (q.min() != null) ? q.removeMin() : -1;

	    System.out.println("    removed value " + ((val == -1 ) ? "null" : val));

	    if (val != vals[inv[i]]) {
		System.out.println("    ...removed incorrect value\n" +
				   "       removed " + ((val == -1 ) ? "null" : val) +  ", expected "
				   + vals[i]);
		return false;	  
	    }

	    if (q.size() != vals.length - i - 1) {
		System.out.println("    ...incorrect size after removing " +
				   val);
		return false;
	    }	    
	}

	return true;
    }

    
    
}
